<?php
// todo 'usar o service'
namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Request;

class Tco01001ElaboracaoCardapio extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $elaboracaoCardapiobaseService;
    protected $cardapiobaseService;
    protected $listas;

    public function __construct($crudService, $parameterBag, $environment, $elaboracaoCardapiobaseService, $cardapiobaseService, $listas ) {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->elaboracaoCardapiobaseService = $elaboracaoCardapiobaseService;
        $this->cardapiobaseService = $cardapiobaseService;
        $this->listas = $listas;
    }

    public function alteraCardapioBase(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $cardapioBaseExistente = $this->elaboracaoCardapiobaseService->retornaCardapioBaseExistente($row);
        if(empty($cardapioBaseExistente) || $row['IDATIVO'] == 'N'){
            if($this->isValid($row, $response)){
                
                $row["__is_new"] = false;
                $row["changed"] = true;
                
                $this->salvaCardapioBase($request);
                $columnsToReturn = $this->crudService->getColumnsToReturn();
                $response->addDataSet(new DataSet("cardapioBase", array()));
                $response->addDataSet(new DataSet("columnsToReturn", $columnsToReturn));
                
            }
        } else {
            $response->addDataSet(new DataSet("cardapioBase", $cardapioBaseExistente));
        }
    }
    
    private function isValid($row, $response) {
        
        if(isset($row["PERIODO"])){
            $dataini = str_replace('/', '-', $row["PERIODO"]["start"]);
            $datafim = str_replace('/', '-', $row["PERIODO"]["end"]);
            
            if(strtotime($dataini) > strtotime($datafim)){
                $response->addMessage(new Message('PERIODO_INVALIDO'));
                return false;
            }
        }
        
        return true;
    }
    
    private function salvaCardapioBase($request) {
        $row = $request->getRow();
        $rows = $this->crudService->getRowsToSave($request, "cardapiobase");
        $this->crudService->save("cardapiobase", $rows);
        $rows[0]["__is_new"] = false;
        $this->crudService->addColumnToReturn($rows[0]);
    }
    
    public function importarCardapioBaseProc(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        if($row["TODAS_CONFIGURACOES"] === 'N') {
            $colunas = "_".implode("_",$row["NMTIPOPRAT"]);
            $row["NRCONFIGCARDBASE"] = $this->listas->criarListaValores($colunas);
        } else {
            $colunas = $row["TODAS_CONFIGURACOES"];
            $row["NRCONFIGCARDBASE"] = $row["TODAS_CONFIGURACOES"];
        }
        try {
            $this->elaboracaoCardapiobaseService->procedureImportarCardBase($row);
            $this->listas->apagarListaValores($colunas);
        } catch (\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }   
    }
    
    public function removePratoCardapioBaseProc(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $row["DSSEMANA"] = "_".implode("_", $row["DSSEMANA"]);
        $row["NMTIPOPRAT"] = "_".implode("_", $row["NMTIPOPRAT"]);
        try {
            $this->elaboracaoCardapiobaseService->procedureRemovePratoCardBase($row);
        } catch (\Exception $e){
            $response->setError(new Error("TCO01000_ERRO_EXECUCAO_PROC_EXCLUSAO_PRATO_CARDBASE", 500));
        }   
    }
    
    public function copiaPratos(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        try {
            $rowCopy = array(
                "NRCARDAPIOBASE"   => $row['origem']["NRCARDAPIOBASE"],
                "NRCOMPCARDBASE"   => $row['origem']["NRCOMPCARDBASE"],
                "NRDIACARDBASE"    => $row['destino']['NRDIACARDBASE'],
                "NRCONFIGCARDBASE" => $row['destino']['NRCONFIGCARDBASE']
            );
            $this->elaboracaoCardapiobaseService->procedureCopiaPratos($rowCopy);
        } catch (\Exception $e){
            throw $e;
        }   
    }
    
    public function replicaPratos(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        try{
            $this->elaboracaoCardapiobaseService->procedureReplicaPratos($row);
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function alteraDiasElaboracaoCardapio(DTO\Request $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrcardapiobase = $row['NRCARDAPIOBASE'];
        $diasExistentes = intval($this->elaboracaoCardapiobaseService->getDiasExistentes($row['NRCARDAPIOBASE']));
        if($row['IDACAO'] == "R") {
            $diasAExcluir = $this->elaboracaoCardapiobaseService->diasAExcluir($nrcardapiobase, $row['DIAS']);
            
            if(($diasExistentes - sizeof($diasAExcluir)) >= 7) {
                //REMOVE REGRA ESPECIFICA
                $this->removeItemAdesaoCardapioBase($diasAExcluir, $response);
                $this->removeAtividadeClienteAdesaoCardapioBase($diasAExcluir, $response);
                $this->removeSegmentoMercadoAdesaoCardapioBase($diasAExcluir, $response);
                $this->removeAdesaoCardapioBase($diasAExcluir, $response);
                //REMOVE PRATOS, COMPOSICAO, REGIAO E DIA
                $this->removeRegioesPratoCompCardapioBase($diasAExcluir, $response);
                $this->removePratosComposicaoCardapioBase($diasAExcluir, $response);
                $this->removeComposicaoCardapioBase($diasAExcluir, $response);
                $this->removeDiaCardapioBase($diasAExcluir, $response);
            } else {
                $response->addDataSet(new DataSet("validacaoDiasCardapio", array("MESSAGECODE" => "TCO01001_VALIDACAO_DIAS_CARDAPIO")));
            }
            
        } else {
            $this->salvaDiacardbase($nrcardapiobase, $row['DIAS']);
        }
        $response->addDataSet(new DataSet("totalDiasPosAlteracao", array("TOTALDIASCARD" => $this->elaboracaoCardapiobaseService->getDiasExistentes($nrcardapiobase))));
    }
    
    private function removeItemAdesaoCardapioBase($rowsDia, $response) {
        
        $rowsToRemove = $this->elaboracaoCardapiobaseService->findItemAdesaoCardapioBase($rowsDia);
        $dataSet = new DataSet("itemadesaocbase", $rowsToRemove);
        $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
        
        $rows = $this->crudService->getRowsToDelete($newRequest, "itemadesaocbase");
        
        $dataSetToDelete = new DataSet("itemadesaocbase", $rows);
		$this->crudService->delete($response, $dataSetToDelete);
    }
    
    private function removeAtividadeClienteAdesaoCardapioBase($rowsDia, $response) {
        
        $rowsToRemove = $this->elaboracaoCardapiobaseService->findAtividadeClienteAdesaoCardapioBase($rowsDia);
        $dataSet = new DataSet("ativcliadesaocbase", $rowsToRemove);
        $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
        
        $rows = $this->crudService->getRowsToDelete($newRequest, "ativcliadesaocbase");
        
        $dataSetToDelete = new DataSet("ativcliadesaocbase", $rows);
		$this->crudService->delete($response, $dataSetToDelete);
    }
    
    private function removeSegmentoMercadoAdesaoCardapioBase($rowsDia, $response) {
        
        $rowsToRemove = $this->elaboracaoCardapiobaseService->findSegmentoMercadoAdesaoCardapioBase($rowsDia);
        $dataSet = new DataSet("segmeradesaocbase", $rowsToRemove);
        $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
        
        $rows = $this->crudService->getRowsToDelete($newRequest, "segmeradesaocbase");
        $dataSetToDelete = new DataSet("segmeradesaocbase", $rows);
		$this->crudService->delete($response, $dataSetToDelete);
    }
    
    private function removeAdesaoCardapioBase($rowsDia, $response) {
        
        $rowsToRemove = $this->elaboracaoCardapiobaseService->findAdesaoCardapioBase($rowsDia);
        $dataSet = new DataSet("adesaocardbase", $rowsToRemove);
        $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
        
        $rows = $this->crudService->getRowsToDelete($newRequest, "adesaocardbase");
        $dataSetToDelete = new DataSet("adesaocardbase", $rows);
		$this->crudService->delete($response, $dataSetToDelete);
    }
    
    private function removeRegioesPratoCompCardapioBase($rowsDia, $response) {
        
        $rowsToRemove = $this->elaboracaoCardapiobaseService->findRegioesPratoComposicaoCardapioBase($rowsDia);
        $dataSet = new DataSet("regiaopratcardbase", $rowsToRemove);
        $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
        
        $rows = $this->crudService->getRowsToDelete($newRequest, "regiaopratcardbase");
        
        $dataSetToDelete = new DataSet("regiaopratcardbase", $rows);
		$this->crudService->delete($response, $dataSetToDelete);
    }
    
    private function removePratosComposicaoCardapioBase($rowsDia, $response) {
        
        $rowsToRemove = $this->elaboracaoCardapiobaseService->findPratosComposicaoCardapioBase($rowsDia);
        $dataSet = new DataSet("pratcompcardbase", $rowsToRemove);
        $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
        
        $rows = $this->crudService->getRowsToDelete($newRequest, "pratcompcardbase");
        
        $dataSetToDelete = new DataSet("pratcompcardbase", $rows);
		$this->crudService->delete($response, $dataSetToDelete);
    }
    
    private function removeComposicaoCardapioBase($rowsDia, $response) {

        $rowsToRemove = $this->elaboracaoCardapiobaseService->findComposicaoCardapioBase($rowsDia);
        $dataSet = new DataSet("compcardbase", $rowsToRemove);
        $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
        
        $rows = $this->crudService->getRowsToDelete($newRequest, "compcardbase");
        
        $dataSetToDelete = new DataSet("compcardbase", $rows);
		$this->crudService->delete($response, $dataSetToDelete);
    }
    
    private function removeDiaCardapioBase($rowsDia, $response) {
        
        $dataSet = new DataSet("diacardbase", $rowsDia);
        $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
        
        $rows = $this->crudService->getRowsToDelete($newRequest, "diacardbase");
        
        $dataSetToDelete = new DataSet("diacardbase", $rows);
		$this->crudService->delete($response, $dataSetToDelete);
    }
    
    private function salvaDiacardbase($nrcardapiobase, $diasAdicionados){
        $qtDiasAdicionados = intval($diasAdicionados);
        $rowsToSave = $this->buildRowsToSaveDiasCardapioBase($nrcardapiobase, $qtDiasAdicionados, 7);
        $dataSet = new DataSet("diacardbase", $rowsToSave);
        $request = new DTO\Request\DataSet($dataSet, "", "", "");
        $rows = $this->crudService->getRowsToSave($request, "diacardbase");
        $this->crudService->save("diacardbase", $rows);
    }
    
    private function buildRowsToSaveDiasCardapioBase($nrcardapiobase,  $qtDiasAdicionados, $daysPerWeek) {
        $semana = $this->elaboracaoCardapiobaseService->getUltimaSemana($nrcardapiobase);
        $qtDiasExistentes = intval($this->elaboracaoCardapiobaseService->getDiasExistentes($nrcardapiobase));
        $rows = array();
        
        $totalDays = $qtDiasExistentes + $qtDiasAdicionados;
        
        for ($i = $qtDiasExistentes+1; $i <= $totalDays; $i++) {
            if($i%$daysPerWeek === 1) {
                $semana++;
            }
            array_push($rows, array(
                "NRCARDAPIOBASE"    => $nrcardapiobase,
                "NRDIASEMANA"       => $i%$daysPerWeek === 0 ? $daysPerWeek : $i%$daysPerWeek,
                "NRSEMANA"          => $semana, 
                "__using_unit"      => false, 
                "__new_code_pk"     => "DIACARDBASE", 
                "__table_pk"        => "NRDIACARDBASE", 
                "__new_code_length" => 1, 
                "__cast_number"     => false, 
                "__is_new"          => true
            ));
        }
        return $rows;
    }
    
    public function validaColunasPadraoServicoFixo(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $resultado = $this->elaboracaoCardapiobaseService->validaColunasPadraoServicoFixo($row);
            $resultado = ['TEM_CONFIG_PADRAO' => (!$resultado) ? false : true];
            $response->addDataSet(new DataSet("cardapiobase", $resultado));
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));    
        }
    }
    
    public function alterarServicoFixo(DTO\Request $request, DTO\Response $response){
        try{
            $row = $request->getRow();
            $this->elaboracaoCardapiobaseService->alterarServicoFixo($row);    
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
}
