<?php

namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource;

class Tco01008CardapiosComplementares extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $crudService;
    protected $environment;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        $this->crudService   = $crudService;
        $this->environment   = $environment;
        $this->entityManager = $entityManager;
    }
    
    public function salvaCnfCComplCardapioBase(DTO\Request $request, DTO\Response $response) {
        $rowsTiposPrato = $request->getDataSet()->getRows();
        
        $rowsToSave = array();
        $rowsToDelete = array();
        
        foreach ($rowsTiposPrato as $row) {
            if($row["TPPRATOINTGRCUSTO"]) {
                if(!isset($row["NRCNFCCOMPLCARDBASE"])){
                    $row["__is_new"] = true;
                }
                $row["changed"] = true;
                $row["__is_new_code"] = true;
                $row["__new_code_pk"] = "CNFCCOMPLCARDBASE";
                $row["__table_pk"] = "NRCNFCCOMPLCARDBASE";
                $row["__new_code_length"] = 1;
                
                array_push($rowsToSave, $row);
            } else {
                array_push($rowsToDelete, $row);
            }
        }
        
        if(sizeof($rowsToSave)) {
            $this->salvaCnfCComplCardBase($rowsToSave);
        } 
        
        if(sizeof($rowsToDelete)) {
            $this->removeCnfCComplCardBase($rowsToDelete, $response);
        }
        
    }
    
    private function salvaCnfCComplCardBase($rowsToSave) {
        $dataSet = new DataSet("cnfccomplcardbase", $rowsToSave);
        $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
        
        $rows = $this->crudService->getRowsToSave($newRequest, "cnfccomplcardbase");
        $this->crudService->save("cnfccomplcardbase", $rows);
    }
    
    private function removeCnfCComplCardBase($rowsToDelete, $response) {
        $dataSet = new DataSet("cnfccomplcardbase", $rowsToDelete);
        $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
        
        $rows = $this->crudService->getRowsToDelete($newRequest, "cnfccomplcardbase");
        $novoDataSet = new DataSet("cnfccomplcardbase", $rows);
		$this->crudService->delete($response, $novoDataSet);
    }
    
    public function salvaCardapiosComplementados(DTO\Request $request, DTO\Response $response) {
        $cardapiosBaseComplementados = $request->getDataSet()->getRows();
        
        $rowsToSave = array();
        
        foreach ($cardapiosBaseComplementados as $row) {
                $row["__is_new_code"] = true;
                $row["__new_code_pk"] = "CARDCOMPLCARDBASE";
                $row["__table_pk"] = "NRCARDCOMPLCARDBASE";
                $row["__new_code_length"] = 1;
                
                array_push($rowsToSave, $row);
        }
        
        $this->salvaCardComplCardBase($rowsToSave);
    }
    
    private function salvaCardComplCardBase($rowsToSave) {
        $dataSet = new DataSet("cardcomplcardbase", $rowsToSave);
        $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
        
        $rows = $this->crudService->getRowsToSave($newRequest, "cardcomplcardbase");
        $this->crudService->save("cardcomplcardbase", $rows);
    }
    
    public function removeCardapiosComplementados(DTO\Request $request, DTO\Response $response) {
        $rowsToDelete = $request->getDataSet()->getRows();
        $cardapiosComplementadosExistentes = $this->retornaCardapiosComplementadosExistentes($rowsToDelete);
        
        if(sizeof($cardapiosComplementadosExistentes) > 0) {
            $this->removeCardComplCardBase($rowsToDelete, $response);
        } else {
            $response->addMessage(new Message('TCO01008_CARDAPIOCOMPL_ZERADO'));
        }
        
    }
    
    private function retornaCardapiosComplementadosExistentes($rows) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT NRCARDCOMPLCARDBASE,
                       NRCARDAPIOBASE,
                       NRCARDAPIOBASECOMPL,
                       NRORG,
                       IDATIVO
                  FROM CARDCOMPLCARDBASE
                 WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE 
                   AND NOT INSTR(:NRCARDAPIOBASECOMPL, NRCARDAPIOBASECOMPL) > 0
                   AND NRORG = :NRORG";
                   
        $query = $connection->prepare($sql);
        
        $cardapiosComplementados = "";

        foreach($rows as $row) {
            $cardapiosComplementados .= "_".$row["NRCARDAPIOBASECOMPL"];
        }

        $params = array(
            "NRCARDAPIOBASE" => $rows[0]["NRCARDAPIOBASE"],
            "NRCARDAPIOBASECOMPL"  => $cardapiosComplementados,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        return $result;
    }
    
    private function removeCardComplCardBase($rowsToDelete, $response) {
        $dataSet = new DataSet("cardcomplcardbase", $rowsToDelete);
        $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
        
        $rows = $this->crudService->getRowsToDelete($newRequest, "cardcomplcardbase");
        
        $dataSetToDelete = new DataSet("cardcomplcardbase", $rows);
		$this->crudService->delete($response, $dataSetToDelete);
    }

}
