<?php
namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\GER\Services\CrudService;
use Doctrine\DBAL\Connection;
use Exception;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DataSource\FilterCriteria;

class Tco02000UtilizacaoProdutosReceitaPropostaComercial extends \Teknisa\GER\Controllers\TeknisaCrud {
  protected $tco02000UtilizacaoProdutosReceitaPropostaComercialService;
  protected $connection;
  protected $environment;
  protected $crudservice;
  protected $parameterbag;
  protected $dataSourceManager;
  protected $tcoUtil;
  
  public function __construct($crudservice, $parameterbag, $entityManager, $environment, $dataSourceManager, $tcoUtil, $tco02000UtilizacaoProdutosReceitaPropostaComercialService){
    $this->crudservice   = $crudservice;
    $this->parameterbag  = $parameterbag;
    $this->entityManager = $entityManager;
    $this->connection    = $entityManager->getConnection();
    $this->environment   = $environment;
    $this->dataSourceManager = $dataSourceManager;
    $this->tcoUtil = $tcoUtil; 
    $this->tco02000UtilizacaoProdutosReceitaPropostaComercialService = $tco02000UtilizacaoProdutosReceitaPropostaComercialService;
    
    parent::__construct($crudservice, $parameterbag);
  }

  public function returnRecipes($request, $response) {
    $row = $request->getRow();
    $recipes = $this->tco02000UtilizacaoProdutosReceitaPropostaComercialService->returnRecipes($row);
    $response->addDataSet(new DataSet('RECIPES', $recipes));
  }
  
  public function deleteSelectedRecipes($request, $response) {
    $rows = $request->getRow();
    
    try {
      $this->entityManager->beginTransaction();
      
      foreach($rows as $row) {
        $this->tco02000UtilizacaoProdutosReceitaPropostaComercialService->deleteSelectedRecipes($row["NRRECEPRATCARDTRAB"]);
      }
      
      $this->entityManager->commit();

    }catch(\Exception $error) {
      $this->entityManager->rollback();
      $response->setError(
        new DTO\Response\Error($error->getMessage(), 1)
      );
    }
    

  }
  
  public function automaticAlteration($request, $response) {
    try {
      $rows = $request->getRow();
      $this->entityManager->beginTransaction();
      
      $nrCardapioTrab = $rows['P_NRCARDAPIOTRAB'];
      $recipeList = $rows['PRODUTOS']; //VALORES DO CHECKED_ROW
      $values = $rows['VALORES'];     /*  OPERACAO,FORMA, NMPRODUTO,QTPRODPRAT*/
      $activeAmount = $rows['P_QTTOUPDATE']; //VALOR DIGITADO NO CAMPO
      $alterationForm = $values["FORMA"];     //SE QUANTIDADE OU PERCENTUAL
      $operationField = $values["OPERACAO"]; //tipo de OPERAÇÂO se adicionar, incluir ou substituir
      
      $novoProduto = (isset($values["CDPRODUTO"])) ? $values["CDPRODUTO"] : "";
      $sgUnidade = (isset($values["SGUNIDADE"])) ? $values["SGUNIDADE"] : "";
      
      $vrAprovganho = 100;
      $vrAprovLimpeza = 100;
      $quantidadeLiquida = 0;
      $quantidadeBruta = 0;
      
      $nrorg = $this->environment->getNrOrgTrab();
      $operator = $this->environment->getCdOperador();
      
      foreach($recipeList as $recipes) {
        $dadosOriginais = $this->tco02000UtilizacaoProdutosReceitaPropostaComercialService->retornaInformacoes($nrCardapioTrab, $recipes["CDFILAUXPRAT"], $recipes["CDPRATO"], $recipes["CDPRODUTO"]);
       
        if($dadosOriginais){
          $vrAprovganho = 100 + $dadosOriginais["VRPEFATCOCCARDTRAB"];
          if($vrAprovganho <= 0) {
            $vrAprovganho = 100;
          }
          
          $vrAprovLimpeza = 100 - $dadosOriginais["VRPEFATCORCARDTRAB"];
          
          if($vrAprovLimpeza <= 0) {
            $vrAprovLimpeza = 100;
          }
          $quantidadeLiquida = $dadosOriginais["QTPRODRECCARDTRAB"];
        } else {
          if($operationField == "S" || $operationField == "A") {
            continue;
          }
        }
        // se for substituica e nao tiver valor, assume qtliquida com valor antigo
        if($operationField == 'I') {
          $vrAprovganho = 100;
          $vrAprovLimpeza = 100;
          
          $quantidadeLiquida = $activeAmount;
          $quantidadeBruta = $activeAmount;
  
        } else if($operationField == "S") {
          if(!empty($activeAmount) && $activeAmount >= 0) {
            if($alterationForm == "P") {
              $quantidadeLiquida = round(($quantidadeLiquida * $activeAmount)/100, 5);
            } else {
              $quantidadeLiquida = round($activeAmount, 5);
            }
          }
        } else if($operationField == "A") {
          $novoProduto = $recipes["CDPRODUTO"];
          
          if($alterationForm == "P") {
            $quantidadeLiquida = round(($quantidadeLiquida * $activeAmount)/100, 5);
          } else {
            $quantidadeLiquida = round($activeAmount, 5);
          }
          
          
        }

        $quantidadeBruta = ($quantidadeLiquida / $vrAprovganho) * 100;
        $quantidadeBruta = round(($quantidadeBruta / $vrAprovLimpeza) * 100, 5);
        
        
        if($operationField == "S" || $operationField == "I") {
          $novoProdutoJaNaReceita = $this->tco02000UtilizacaoProdutosReceitaPropostaComercialService->retornaInformacoes($nrCardapioTrab, $recipes["CDFILAUXPRAT"], $recipes["CDPRATO"], $novoProduto);
          if($novoProdutoJaNaReceita) {
            if($operationField == "S"){
              throw new \Exception('PRODUTO_JA_EXISTE_DESTINO');
            }
            else{
              continue;
            }
          }
        }

        if($quantidadeLiquida < 0.00001 || $quantidadeBruta < 0.00001){
            throw new \Exception('ERRO_QTD_LIMITE_MINIMO');
        }
        if($operationField == 'I'){
          $this->tco02000UtilizacaoProdutosReceitaPropostaComercialService->insertRecipe($nrorg, $novoProduto, $quantidadeLiquida, $quantidadeBruta, $recipes["CDPRATO"], $recipes["CDFILAUXPRAT"], $nrCardapioTrab, $operator, $sgUnidade);
        } else {
          $this->tco02000UtilizacaoProdutosReceitaPropostaComercialService->updateProductAmount($recipes["CDPRODUTO"] ,$novoProduto, $quantidadeLiquida, $quantidadeBruta, $recipes["CDPRATO"], $recipes["CDFILAUXPRAT"], $nrCardapioTrab, $operator);
        }
      }
      $this->entityManager->commit();

    }catch(\Exception $error) {
      $this->entityManager->rollback();
      $response->setError(
        new DTO\Response\Error($error->getMessage(), 1)
      );
    }
  }
  
  
}